# Functions for Nagios Confgiuration

use HTML::Entities;

do '../web-lib.pl';
&init_config();

do '../ui-lib.pl';

#check all the packages in parameter
#There is only one parameter, which is a long string including all packages' name
#return 1 means all packages have installed
#return 0 means one of them have not installed
sub check_packages_one_parameter() {

	# split th long string to a array
	local @packages = split( / /, $_[0] );

	#print @packages, "<br>";
	return &check_packages(@packages);

}

#check all the packages in parameter
#return 1 means all packages have installed
#return 0 means one of them have not installed
sub check_packages() {
	$result = 1;
	foreach (@_) {
		$result = check_one_package($_);
		if ( $result == 0 ) {
			return 0;
		}
	}
	1;
}

#check only one package if it have installed
#return 1 means installed
#return 0 means not installed
sub check_one_package() {

	#get parameter
	local $package_name = $_[0];

	#run command to check
	local $out = &backquote_command( "dpkg-query -s $package_name", 1 );

	#split each line
	local @lines = split( /\r?\n/, $out );

	#find string "Status: install ok installed"
	foreach (@lines) {
		if ( $text{'dpkg_installed'} eq $_ ) {
			return 1;
		}
	}

	#return result
	#test
	#print "$_[0] NOT installed<br>";
	0;
}

#check nagios service
#return 1 means service is running
#return 0 means service is not running
sub check_nagios_service {

	#run command to check nagios if it is running
	local $out = &backquote_command( "ps aux | grep nagios3", 1 );

	#split each line
	local @lines = split( /\r?\n/, $out );

	#find string "/usr/sbin/nagios3 -d"
	foreach (@lines) {
		if (/\/usr\/sbin\/nagios3 -d/) {
			return 1;    #running
		}
	}

	0;                   #not running
}

# add source to sources.list
sub add_source() {
	local $filename = "/etc/apt/sources.list";

	# add new source in sources.list
	local $command = "printf \"\\n$text{'source_nagios3'}\\n\" >> $filename";
	local $out = &backquote_command( $command, 1 );

}

#copy file sub
sub copy_file() {
	local $source_file      = $_[0];
	local $destination_file = $_[1];

	local $command = "cp $source_file $destination_file";
	local @out = &backquote_command( $command, 1 );

	#print $out, "<br>";
}

#change file owner(chown)
sub nagios_chown() {
	local $owner     = $_[0];
	local $file_name = $_[1];

	local $command = "chown $owner $file_name";
	print $command, "<br>";
	local @out = &backquote_command( $command, 1 );
	print @out, "<br>";
}

#change file premission(chmod)
sub nagios_chmod() {
	local $mode      = $_[0];
	local $file_name = $_[1];

	local $command = "chmod $mode $file_name";
	local @out = &backquote_command( $command, 1 );
	print @out, "<br>";
}

#add some key for new source
sub add_key() {
	local $command1 =
	  "gpg --keyserver wwwkeys.eu.pgp.net --recv-keys EA8E8B2116BA136";
	local $command2 = "gpg --armor --export EA8E8B2116BA136C | apt-key add -";
	local @out1     = &backquote_command( $command1, 1 );
	local @out2     = &backquote_command( $command2, 1 );

	#test
	#	print @out1, "<br>";
	#	print @out2, "<br>";
}

#update package list(apt-get update)
sub update_package_list() {
	local @out = &backquote_command( "apt-get update", 1 );

	#print encode_entities(@out), "<br>";
	print @out, "<br>";
}

#install one package - NOT USE
sub install_package() {
	local $package_name = $_[0];

	local @out =
	  &backquote_command( "apt-get -y --force-yes -f install $package_name",
		1 );
	print @out, "<br>";
}

#install ndo	-- NOT USE
sub install_ndo() {
	local $command_line = $_[0];

	local @out = &backquote_command( "apt-get $command_line", 1 );
	print @out, "<br>";
}

#check centreon
sub check_centreon() {
	return &check_folders(
		$text{'folder_centreon_bin'},
		$text{'folder_centreon_cron'},
		$text{'folder_centreon_doc'},
		$text{'folder_centreon_examples'},
		$text{'folder_centreon_filesGeneration'},
		$text{'folder_centreon_filesUpload'},
		$text{'folder_centreon_GPL_LIB'},
		$text{'folder_centreon_lib'},
		$text{'folder_centreon_log'},
		$text{'folder_centreon_www'}
	);
}

#check folders
#return 1 means exist
#return 0 means not exist
sub check_folders() {
	local $result = 1;
	foreach (@_) {
		$result = &check_one_folder($_);
		if ( $result == 0 ) {
			return 0;
		}
	}
	1;
}

#check one folder
#return 1 means exist
#return 0 means not exist
sub check_one_folder {
	if ( -e $_[0] ) {
		return 1;
	}
	0;
}

#link ndomod
sub link_ndomod() {
	local @out = &backquote_command(
		"ln -s /usr/lib/ndoutils/ndomod-mysql-3x.o /usr/sbin/ndomod.o", 1 );
}

#set password for ndoutils
#parameter 0: username -- ndoutils
#parameter 1: password -- ndouitls
#parameter 2: password of root user -- abc or EMPTY
#
sub set_mysql_password() {
	local $username    = $_[0];
	local $passwd      = $_[1];
	local $passwd_root = $_[2];

	local $command = "mysql -u root ";
	if ( $passwd_root == undef ) {

	}
	else {
		$command = $command . "-p$passwd_root ";
	}

	$command =
	    $command
	  . "-e \"set password for $username"
	  . '@localhost'
	  . "=PASSWORD('$passwd');\"";

	local @out = &backquote_command( $command, 1 );

#mysql -u root -pxxx -e set password for 'ndoutils'@'localhost'=PASSWORD('ndoutils');
}

#install centreon
sub install_centreon_temp() {

	#local @out = &backquote_command( $command, 1 );
	local @out;

	#copy installation file
	print "Copying Centreon installation file to localhost ...<br>";
	&copy_file( "files/centreon-2.0-RC7.tar.gz", "/usr/local/src/" );

	#untar installation file
	print "Extracting Centreon installation file ...<br>";
	@out = &backquote_command(
		"tar zxf /usr/local/src/centreon-2.0-RC7.tar.gz -C /usr/local/src/",
		1 );

	#upgrade PEAR
	print "Upgrading PEAR ...<br>";
	@out = &backquote_command( "pear upgrade pear", 1 );
	print "Installing needed PEAR modules ...<br>";
	@out = &backquote_command(
"pear install -o -f --alldeps DB_DataObject DB_DataObject_FormBuilder MDB2 Numbers_Roman Numbers_Words HTML_Common HTML_QuickForm HTML_QuickForm_advmultiselect HTML_Table Auth_SASL HTTP Image_Canvas Image_Color Image_Graph Image_GraphViz Net_Traceroute Net_Ping Validate XML_RPC SOAP LOG",
		1
	);

	#install.sh
	print "Instaling Centreon ...<br>";
	@out = &backquote_command(
		"/usr/local/src/centreon-2.0-RC7/install.sh -f files/sample.tmpl", 1 );

}

#check whether finish the web installation
#return 0 means web installation not finish
#return 1 means web installation has been done
sub check_webinstallation() {
	local $DB_name = undef;
	local $result  = undef;

	#check conf.pm
	if ( -e "/etc/centreon/conf.pm" ) {

		#get DB name from the file
		$DB_name =
		  &get_DB_names( "/etc/centreon/conf.pm", "mysql_database_oreon" );

		#test
		#print "get DB name: $DB_name <br>";

		if ( $DB_name ne undef ) {
			$result = &check_one_DB($DB_name);

			#test
			#print "result is $result <br>";

			if ( $result == 0 ) {
				return 0;
			}
		}
		else {
			return 0;
		}

		#reset DB_name
		$DB_name = undef;

		#get DB name from the file
		$DB_name =
		  &get_DB_names( "/etc/centreon/conf.pm", "mysql_database_ods" );
		if ( $DB_name ne undef ) {
			$result = &check_one_DB($DB_name);
			if ( $result == 0 ) {
				return 0;
			}
		}
		else {
			return 0;
		}

		return 1;
	}
	else {
		return 0;
	}

}

#get DB names from conf.pm
#parameter 0: full path of file
#parameter 1: DB type
#return DB name or 0
sub get_DB_names() {
	local $file_path = $_[0];
	local $DB_type   = $_[1];

	local $line;
	local @db_line;

	#local $db_num;

	if ( open( $hfile, $file_path ) ) {
		while ( $line = <$hfile> ) {

			#print $line,"<br>";
			if ( $line =~ /$DB_type/ ) {

				#print $line,"!!!!!!!!!!<br>";
				@db_line = split( /=/, $line );

				#print @db_line,"<br>";
				#$db_num = @db_line;
				#print $db_num,"<br>";
				#print "_",$db_line[1],"_<br>";
				$db_line[1] =~ s/"//g;
				$db_line[1] =~ s/;//g;
				$db_line[1] =~ s/ //g;

				#print "_",$db_line[1],"_<br>";
				return $db_line[1];
			}
		}
	}
	else {
		return undef;
	}

}

#check DB exist
# parameter 0: DB name
# parameter 1: root password
# return 0:	DB not exist
# return 1: DB exist
sub check_one_DB() {
	local $DBname      = $_[0];
	local $passwd_root = $_[1];

	#test
	#print "checking DB:$DBname ...<br>";

	local $command = "mysql -u root ";

	if ( $passwd_root == undef ) {
	}
	else {
		$command = $command . "-p$passwd_root ";
	}

	$command = $command . "-e \"use $DBname;\"";

	local @out = &backquote_command( $command, 1 );

	#find error in output
	foreach (@out) {
		if (/ERROR 1049/) {
			return 0;    #DB not exist
		}
	}

	1;
}

#copy htpasswd.users to /etc/nagios3
# if the file already exist, do nothing
sub copy_htpasswd() {
	if ( -e "/etc/nagios3/htpasswd.users" ) {

	}
	else {

		#copy file
		&copy_file( "files/htpasswd.users", "/etc/nagios3/htpasswd.users" );
	}
}

#check_DB
sub check_db() {
	local @out;
	local $result      = undef;
	local $command     = undef;
	local $command_sql = undef;

	#print "checking db ...<br>";

	#get DB names
	local $DB_name =
	  &get_DB_names( "/etc/centreon/conf.pm", "mysql_database_oreon" );

	#check DB by bash
	$command = "files/checkDB.sh $DB_name";

	#get output
	@out = &backquote_command( $command, 1 );

	chomp( $out[1] );

	#check name
	if ( $out[1] eq "ndoutils" ) {
		return 1;
	}
	else {
		return 0;
	}

}

#exec sql
#parameter 0: sql sentence
#parameter 1: password of root
sub exec_sql() {
	local $sql         = $_[0];
	local $passwd_root = $_[1];

	local $command = "mysql -u root ";

	if ( defined($passwd_root) ) {
		$command = $command . "-p$passwd_root ";
	}

	$command = $command . "-e \"" . $sql . "\"";

	#test
	print $command, "<br>";
	local @out = &backquote_command( $command, 1 );
}

#change DB centreon
#parameter 0: BD name
sub change_db_centreon
{
	local $DB_name = $_[0];
	local $passwd = "abc";
	#local $command = "UPDATE $DB_name.cfg_ndo2db SET db_name=\'ndoutils\', db_user= \'ndoutils\',db_pass= \'ndoutils\';";
	
	&exec_sql("UPDATE $DB_name.cfg_ndo2db SET db_name=\'ndoutils\', db_user= \'ndoutils\',db_pass= \'ndoutils\';");
	&exec_sql("UPDATE $DB_name.cfg_nagios SET object_cache_file='/var/cache/nagios3/objects.cache';");
	&exec_sql("UPDATE $DB_name.general_opt SET nagios_path_img = '/usr/share/nagios3/htdocs/images/logos/';");
	&exec_sql("UPDATE $DB_name.general_opt SET nagios_path_bin = '/usr/sbin/nagios3';");
	&exec_sql("UPDATE $DB_name.cfg_cgi SET main_config_file = '/etc/nagios3/nagios.cfg';");
	&exec_sql("UPDATE $DB_name.cfg_cgi SET physical_html_path = '/usr/share/nagios3/htdocs';");
	&exec_sql("UPDATE $DB_name.cfg_cgi SET url_html_path = '/nagios3';");
	&exec_sql("UPDATE $DB_name.cfg_cgi SET nagios_check_command = '/usr/lib/nagios/plugins/check_nagios /var/cache/nagios3/status.dat 5 /usr/sbin/nagios3';",$passwd);
	&exec_sql("UPDATE $DB_name.cfg_nagios SET log_file = '/var/log/nagios3/nagios.log';");
	&exec_sql("UPDATE $DB_name.cfg_nagios SET cfg_dir = '/etc/nagios3/';");
	&exec_sql("UPDATE $DB_name.cfg_nagios SET temp_file = '/var/tmp/nagios.tmp';");
	&exec_sql("UPDATE $DB_name.cfg_nagios SET status_file = '/var/cache/nagios3/status.dat';");
	&exec_sql("UPDATE $DB_name.cfg_nagios SET p1_file = '/usr/lib/nagios3/p1.pl';");
	&exec_sql("UPDATE $DB_name.cfg_nagios SET log_archive_path = '/var/log/nagios3/archives/';");
	&exec_sql("UPDATE $DB_name.cfg_nagios SET command_file = '/var/lib/nagios3/rw/nagios.cmd';");
	&exec_sql("UPDATE $DB_name.cfg_nagios SET downtime_file = '/var/lib/nagios3/downtime.dat';");
	&exec_sql("UPDATE $DB_name.cfg_nagios SET comment_file = '/var/lib/nagios3/comments.dat';");
	&exec_sql("UPDATE $DB_name.cfg_nagios SET lock_file = '/var/run/nagios3/nagios3.pid';");
	&exec_sql("UPDATE $DB_name.cfg_nagios SET state_retention_file = '/var/lib/nagios3/retention.dat';");
}

#change DB centstorage
#parameter 0: BD name
sub change_db_centstorage() 
{
	local $DB_name = $_[0];
	local $passwd = "abc";
	&exec_sql ("UPDATE $DB_name.config SET drop_file = '/var/log/nagios3/service-perfdata.tmp';");
	&exec_sql ("UPDATE $DB_name.config SET perfdata_file = '/var/log/nagios3/service-perfdata';");
	&exec_sql ("UPDATE $DB_name.config SET nagios_log_file = '/var/log/nagios3/nagios.log';");

}
